print("import: 'poetry'")
import poetry

print("import: 'poetry.console'")
import poetry.console

print("import: 'poetry.console.commands'")
import poetry.console.commands

print("import: 'poetry.console.commands.cache'")
import poetry.console.commands.cache

print("import: 'poetry.console.commands.debug'")
import poetry.console.commands.debug

print("import: 'poetry.console.commands.self'")
import poetry.console.commands.self

print("import: 'poetry.installation'")
import poetry.installation

print("import: 'poetry.io'")
import poetry.io

print("import: 'poetry.json'")
import poetry.json

print("import: 'poetry.layouts'")
import poetry.layouts

print("import: 'poetry.masonry'")
import poetry.masonry

print("import: 'poetry.masonry.builders'")
import poetry.masonry.builders

print("import: 'poetry.masonry.publishing'")
import poetry.masonry.publishing

print("import: 'poetry.masonry.utils'")
import poetry.masonry.utils

print("import: 'poetry.mixology'")
import poetry.mixology

print("import: 'poetry.packages'")
import poetry.packages

print("import: 'poetry.packages.constraints'")
import poetry.packages.constraints

print("import: 'poetry.packages.utils'")
import poetry.packages.utils

print("import: 'poetry.puzzle'")
import poetry.puzzle

print("import: 'poetry.puzzle.operations'")
import poetry.puzzle.operations

print("import: 'poetry.repositories'")
import poetry.repositories

print("import: 'poetry.semver'")
import poetry.semver

print("import: 'poetry.spdx'")
import poetry.spdx

print("import: 'poetry.utils'")
import poetry.utils

print("import: 'poetry.vcs'")
import poetry.vcs

print("import: 'poetry.version'")
import poetry.version

